;***************************************************************************
;*   Library Name: Lasers
;*       Coded By: John Logsdon
;*   Last Updated: 8/5/2012
;*        Purpose: Handles all of the laser procedures
;***************************************************************************

;*************************************************************
; PROCEDURE: LoadLaserFrames(...)
;
; This function moves the player's character by a certain
; number of pixels.
;
; Arguments: 
;   Laser_Full_Path: File name, including full path
;   LaserWidth:  How wide the image is
;   LaserHeight: how high the image is
;   LaserSpacer: how many pixels between each image in the file
;
; Returns:
;   0 = Everything loaded fine
;   ? = Load failed
;*************************************************************
Procedure LoadLaserFrames(Laser_Full_Path.s,LaserWidth.l,LaserHeight.l,LaserSpacer.l)

  Dim LaserImages.Laser_Frame_Images(0)
  
  ; first off let's load the full image containing all the tiles
  ; into a temporary space
  Temp_Image.l = LoadSprite(#PB_Any,Laser_Full_Path.s)
  
  ImageSizeX.l = SpriteWidth(Temp_Image.l)
  ImageSizeY.l = SpriteHeight(Temp_Image.l)
  
  LaserColumns.l = ImageSizeX.l / LaserWidth.l
  LaserRows.l = ImageSizeY.l / LaserHeight.l
  
  Dim LaserImages.Laser_Frame_Images(LaserRows.l * LaserColumns.l)
  
  If Temp_Image.l = 0
    ProcedureReturn(-1)
  EndIf
  
  ; get the current buffer so we can restore to it
  DisplaySprite(Temp_Image,0,0)

  
  ; setup our basic vars.  The X and Y values will be whatever the XSpacer
  ; and YSpacer values are. If they are 0, then it's assumed that there is
  ; no space between the tiles. If you look at the sample images included
  ; with this demo you'll see a white box around each image...that's why
  ; I have these spacers...we don't wanna load the boxes, just the images.
  X.l = LaserSpacer.l
  Y.l = LaserSpacer.l
  
  ; keep track of the number of images
  ImageNumber.l = 0
  
  ; run through the total number of rows (minus 1, of course)
  For Rows.l = 0 To LaserRows.l -1
    ; and run through all the columns per row (minus 1 again)
    For Columns.l = 0 To LaserColumns.l -1
      ; create a new TileList element and assign it the current
      ; ImageNumber.  Then populate the TileList element with
      ; the TileWidth, TileHeight, and the actual Tile_Image
      LaserImages(ImageNumber)\Width = LaserWidth
      LaserImages(ImageNumber)\Height = LaserHeight
      
      ; then we grab the image based off the size setup in CreateImage(...)
      ; from our X,Y location
      LaserImages(ImageNumber)\Image=GrabSprite(#PB_Any,X,Y,LaserWidth,LaserHeight)
      
      ; now we add X to the TileWidth and the XSpacer to get the new X position.
      ; So, if our current X = 2, and the TileWidth = 32 and the XSpacer = 2, we'd
      ; have X = 2 + 32 + 2, or 36.  This means that the next time we call         
      ; GrabImage(...) it will start grabbing from the X position 36 (or the 36th pixel 
      ; from the left).         
      X.l = X.l + LaserWidth.l + LaserSpacer.l
      
      ; increment our tile counter (for the array positioning)
      ImageNumber.l = ImageNumber.l + 1
    Next
    ; we've finished with that row, so reset X back to the spacer position
    X.l = LaserSpacer.l
    
    ; now we add Y to the TileHeight and the YSpacer to get the new Y position.
    ; So, if our current Y = 2, and the TileHeight = 32 and the YSpacer = 2, we'd
    ; have Y = 2 + 32 + 2, or 36.  This means that the next time we call  
    ; GrabImage(...) it will start grabbing from the Y position 36 (or the 36th pixel 
    ; from the top).         
    Y.l = Y.l + LaserHeight.l + LaserSpacer.l
  Next
  
  ; free the image from memory, so we don't hold memory for no reason
  FreeSprite(Temp_Image)
  
  ProcedureReturn(0)
EndProcedure



;*************************************************************
; PROCEDURE: AddLaser(...)
;
; This function adds a laser to the Laser() list
;
; Arguments: 
;   Owner: 0 = player, 1 = robot
;   X: X in pixels (worldX)
;   Y: Y in pixels (worldy
;   WX: Copy of X
;   WX: Copy of Y
;   Direction: Which direction is it moving?
;   TileWidth: how wide are the images
;   TileHeight: how high are the images
;
; Returns:
;   N/A
;*************************************************************
Procedure AddLaser(Owner.l,X.l,Y.l,WX.l,WY.l,Direction.l,TileWidth,TileHeight)
  ; Add it to the ladder Type
  AddElement(Laser())
  Laser()\Owner = Owner 
  Laser()\WorldX = WX
  Laser()\WorldY = WY
  Laser()\ScreenX = X
  Laser()\ScreenY = Y
  Laser()\Direction = Direction
  Laser()\Distance = 0
  Laser()\Status = 0
  Laser()\Timer = ElapsedMilliseconds()
  Laser()\Timeout = 10
  If Owner = 0
     MyPlaySound(1)
     Laser()\Beam = 0
     Laser()\Explosion = 1
  Else
     MyPlaySound(2)
     Laser()\Beam = 2
     Laser()\Explosion = 3
  EndIf
EndProcedure


;*************************************************************
; PROCEDURE: MoveLasers(...)
;
; This function moves lasers around, sees if they hit anything
; and makes a determination as to what was hit and what to do.
;
; Arguments: 
;   Current_Time: last call to elapsedmilliseconds()
;   MoveMap: Is the map moving?
;   Direction: What direction is it moving?
;   TileWidth: how wide are the tiles
;   TileHeight: how high are the tiles
;
; Returns:
;   N/A
;*************************************************************
Procedure MoveLasers(Current_Time,MoveMap.l,Direction.l,TileWidth,TileHeight)
  
  ; Run through the lasers
  ForEach(Laser())
     ; if this laser is ready to move
     If Current_Time > Laser()\Timer + Laser()\Timeout
        Select Laser()\Direction
           Case 1   ; Left
                  ; save the old X position
                  OldX = Laser()\WorldX
                  ; move the laser by LaserSpeed
                  Laser()\WorldX = Laser()\WorldX - LaserSpeed
                  ; use some quick var assignments to save on typing
                  X = Laser()\WorldX
                  Y = Laser()\WorldY
                  ; check to see if the laser has hit a wall
                  If Map_CheckWallCollisionManual(X,Y,X+8,Y+8,TileWidth,TileHeight) = 1
                     ; if so, reset the laser position
                     Laser()\WorldX = OldX
                     ; and then up the status by one so it goes into the explode phase
                     Laser()\Status = Laser()\Status + 1
                  Else ; if we haven't hit a wall
                     ; see if the laser was fired by the player
                     If Laser()\Owner = 0
                        ; if so, run through all the robots
                        ForEach(Robot())
                          ; and see if there's been any hits
                          If SpritePixelCollision(RobotImages(Robot()\Frame)\Image, Robot()\WorldX, Robot()\WorldY, LaserImages(Laser()\Beam)\Image, X, Y) 
                             ; if so, make sure it's not just the previous hit still exploding
                             If Robot()\Hit <= 0
                                ; it's a new hit, so play a sound and set the hit variables all up
                                MyPlaySound(5)
                                Robot()\Hit = 3
                                Robot()\Firing = 0
                                Robot()\ActionTimer = Current_Time
                                Robot()\ActionTimeout = 500
                                ; yank some off the Robot's armor
                                Robot()\Armor = Robot()\Armor - RobotArmorHit
                                ; if the robot's armor is depleted...bye bye, robot :(
                                If Robot()\Armor < 5 
                                   ; play the dying robot sound and flag it for dead
                                   MyPlaySound(6)
                                   Robot()\Dead = 3
                                EndIf
                                ; set up the appropriate HIT frame here
                                If Robot()\Direction = 1 Or Robot()\Direction = 3
                                   Robot()\Frame = 55
                                Else
                                   Robot()\Frame = 48
                                EndIf
                             EndIf
                             ; set up our flags for when we're outside of the Robot() loop
                             RobotHit = Robot()\Hit
                             RobotDead = Robot()\Dead
                             ; hit's been made, so no since continuing inside this loop, break out
                             Break
                          EndIf
                        Next
                     Else ; the laser belongs to the robot
                        ; see if the laser has hit the player
                        If SpritePixelCollision(Player(PlayerFrame)\Image,PlayerWorldX,PlayerWorldY, LaserImages(Laser()\Beam)\Image, X, Y)
                           ; if so, increase the laser status to start the explode phase
                           Laser()\Status = Laser()\Status + 1     
                           ; save the laser's X postion...just in case
                           Laser()\WorldX = OldX
                           
                           ; make sure that the player hasn't already been hit and that we don't show the explosion over and over
                           If PlayerHit <= 0 And Laser()\Status < 2
                              ; shut off any prior firing attempts by the player...he just got shot, he's not gonna fire yet
                              PlayerFiring = 0
                              ; shave off the armor 
                              PlayerArmor = PlayerArmor - PlayerArmorHit
                              ; drop the health bar
                              PlayerHealthBar = PlayerHealthBar - 3
                              ; if the player is under 10 armor, he's dead
                              If PlayerArmor < 10
                                 ; play the dead sound and flag the player as a goner
                                 MyPlaySound(4)
                                 PlayerDead = 1
                                 ; set the appropriate frame
                                 If PlayerFacing = 1 Or PlayerFacing = 3
                                    PlayerFrame = 95
                                 Else
                                    PlayerFrame = 88
                                 EndIf
                                 ; set the restart timer
                                 RestartTime = 6
                              Else ; player's not dead, just hit
                                 ; play the sound and set the hit flags accordingly
                                 MyPlaySound(3)
                                 PlayerHit = 1
                                 PlayerHitAnimation = 3
                              EndIf
                           EndIf
                        EndIf
                     EndIf
                      
                     ; now that we're out of the robots loop, let's see where we are
                     ; if the laser is player fired
                     If Laser()\Owner = 0
                        ; and the robot is neither hit nor dead
                        If RobotHit = 0 And RobotDead = 0
                           ; keep'r movin
                           Laser()\ScreenX = Laser()\ScreenX - LaserSpeed
                        Else
                           ; if the robot is dead, set the laser status to 99 so it'll get deleted
                           If RobotDead > 0
                              Laser()\Status = 99
                           Else ; if the robot has just been hit
                              ; reset the laser position
                              Laser()\WorldX = OldX
                              ; and start the explosion going
                              Laser()\Status = Laser()\Status + 1     
                           EndIf
                        EndIf
                     Else ; robot fired laser
                        ; if the player's not been hit
                        If PlayerHit = 0 And PlayerDead = 0
                           ; keep'r movin
                           Laser()\ScreenX = Laser()\ScreenX - LaserSpeed
                        Else
                           ; if the player is dead, set the laser status to 99 so it'll get deleted
                           If PlayerDead > 0
                              Laser()\Status = 99
                           Else ; the player's just been hit
                              ; reset the laser position
                              Laser()\WorldX = OldX
                              ; start the laser exploding phase
                              Laser()\Status = Laser()\Status + 1     
                              ; reset the player hit var
                              PlayerHit = 0
                           EndIf
                        EndIf
                     EndIf
                  EndIf

           ;;;;;;;;;;;follow the comments up in Case 1 which are nearly identical to these, except for direction
           Case 2   ; Right
                  OldX = Laser()\WorldX
                  Laser()\WorldX = Laser()\WorldX + LaserSpeed
                  X = Laser()\WorldX
                  Y = Laser()\WorldY
                  If Map_CheckWallCollisionManual(X,Y,X+8,Y+8,TileWidth,TileHeight) = 1
                     Laser()\Status = Laser()\Status + 1
                     Laser()\WorldX = OldX
                  Else
                     If Laser()\Owner = 0
                        ForEach(Robot())
                          If SpritePixelCollision(RobotImages(Robot()\Frame)\Image, Robot()\WorldX, Robot()\WorldY, LaserImages(Laser()\Beam)\Image, X, Y) 
                             If Robot()\Hit <= 0
                                MyPlaySound(5)
                                Robot()\Hit = 3
                                Robot()\Firing = 0
                                Robot()\ActionTimer = Current_Time
                                Robot()\ActionTimeout = 500
                                Robot()\Armor = Robot()\Armor - RobotArmorHit
                                If Robot()\Armor < 5 
                                   MyPlaySound(6)
                                   Robot()\Dead = 3
                                EndIf
                                If Robot()\Direction = 1 Or Robot()\Direction = 3
                                   Robot()\Frame = 55
                                Else
                                   Robot()\Frame = 48
                                EndIf
                             EndIf
                             RobotHit = Robot()\Hit
                             RobotDead = Robot()\Dead
                             Break
                          EndIf
                        Next
                     Else
                        If SpritePixelCollision(Player(PlayerFrame)\Image,PlayerWorldX ,PlayerWorldY, LaserImages(Laser()\Beam)\Image, X, Y)
                           Laser()\Status = Laser()\Status + 1     
                           Laser()\WorldX = OldX
                           If PlayerHit <= 0 And Laser()\Status < 2
                              PlayerFiring = 0
                              PlayerArmor = PlayerArmor - PlayerArmorHit
                              PlayerHealthBar = PlayerHealthBar - 3
                              If PlayerArmor < 10
                                 MyPlaySound(4)
                                 PlayerDead = 1
                                 If PlayerFacing = 1 Or PlayerFacing = 3
                                    PlayerFrame = 95
                                 Else
                                    PlayerFrame = 88
                                 EndIf
                                 RestartTime = 6
                              Else
                                 MyPlaySound(3)
                                 PlayerHit = 0
                                 PlayerHitAnimation = 3
                              EndIf
                           EndIf
                        EndIf
                     EndIf
                  
                     If Laser()\Owner = 0
                        If RobotHit = 0 And RobotDead = 0
                           Laser()\ScreenX = Laser()\ScreenX + LaserSpeed
                        Else
                           If RobotDead > 0
                              Laser()\Status = 99
                           Else
                              Laser()\WorldX = OldX
                              Laser()\Status = Laser()\Status + 1     
                           EndIf
                        EndIf
                     Else
                        If PlayerHit = 0 And PlayerDead = 0
                           Laser()\ScreenX = Laser()\ScreenX + LaserSpeed
                        Else
                           If PlayerDead > 0
                              Laser()\Status = 99
                           Else
                              Laser()\WorldX = OldX
                              Laser()\Status = Laser()\Status + 1  
                              PlayerHit = 0
                           EndIf
                        EndIf
                     EndIf
                  EndIf
        EndSelect
        ; reset the laser timer
        Laser()\Timer = Current_Time
     EndIf

     ; if the map is moving we need to compensate by moving the laser
     ; the same distance that the map is scrolling, in whichever direction
     ; is relavent.
     If MoveMap.l = 1
        Select Direction 
           Case 1
               Laser()\ScreenY = Laser()\ScreenY + #MoveDistance
           Case 2
               Laser()\ScreenY = Laser()\ScreenY - #MoveDistance
           Case 3
               Laser()\ScreenX = Laser()\ScreenX + #MoveDistance
           Case 4
               Laser()\ScreenX = Laser()\ScreenX - #MoveDistance
        EndSelect
     EndIf

     ; update the laser's distance information
     Laser()\Distance = Laser()\Distance + 1
     ; if the laser is still in good shape and still has some distance to go, draw it
     If Laser()\Status = 0 And Laser()\Distance < AllowableDistance
        DisplayTransparentSprite(LaserImages(Laser()\Beam)\Image,Laser()\ScreenX,Laser()\ScreenY)
     Else
        ; if the laser has hit something, draw the explosion
        If Laser()\Status < 5 And Laser()\Distance < AllowableDistance
           DisplayTransparentSprite(LaserImages(Laser()\Explosion)\Image,Laser()\ScreenX,Laser()\ScreenY)
        Else
           ; if the laser has passed it's allowable distance, or the explosion animation is over, delete it
           DeleteElement(Laser())
        EndIf
     EndIf
  Next
EndProcedure

;*************************************************************
; PROCEDURE: DeleteLasers(...)
;
; Nukes all the lasers from the list
;
;*************************************************************
Procedure DeleteLasers()
  ForEach(Laser())
     DeleteElement(Laser())
  Next
EndProcedure 
; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 420
; FirstLine = 392
; Folding = -