;***************************************************************************
;*   Library Name: MapLib
;*       Coded By: John Logsdon
;*   Last Updated: 4/14/2005
;*        Purpose: To allow the loading of tiles and maps for display and
;*                 and scrolling.  Handles both Text and Binary, but does
;*                 NOT do encryption.
;*                 
;*                 Also now includes the collision information and checking. 
;***************************************************************************


;*************************************************************************
;* Begin Map Control Defines
;*************************************************************************

; Structure For the TileList
Structure Map_Tiles
  Image.l
  X.l
  Y.l
  Width.l
  Height.l
EndStructure

Global Map_TotalTiles.l
Map_TotalTiles = 0

Global Map_TilesImages.l

; Dimension our tile array
Global Dim Tile.Map_Tiles(Map_TotalTiles)
Global Map_FullTiles.l
Map_FullTiles = 0

; Structure for the Map_Data
Structure Map_Data
 TileNumber.l
EndStructure

; Globals to track the map dimensions
Global Map_Width = 1
Global Map_Height = 1
Global Map_X_Start = 0
Global Map_Y_Start = 0
Global Map_X_DisplayOffset = 0
Global Map_Y_DisplayOffset = 0
Global Map_ScrollWidth = 0
Global Map_ScrollHeight = 0

; Dimension our Map array
Global Dim Map.Map_Data(Map_Width,Map_Height)
Global Dim MapHold.Map_Data(Map_Width,Map_Height)

;*************************************************************************
;* End Map Control Defines
;*************************************************************************
;
;
;
;*************************************************************************
;* Begin Collision Control Defines
;*************************************************************************

; create the Player collision array for bounding boxes on the player
Global Dim Map_PlayerCollisionArray(16)

; setup the walls structure for knowing where all the walls are for collisions
Structure Map_Walls
 TileNumber.l
 X.l
 Y.l
EndStructure

; setup a list for the map walls
Global NewList Wall.Map_Walls()

; setup the walls structure for knowing where all the walls are for collisions
Structure Map_Ladders
  TileNumber.l
  X.l
  Y.l
EndStructure

; setup a list for the map walls
Global NewList Ladder.Map_Ladders()


;*************************************************************************
;* End Collision Control Defines
;*************************************************************************
;
;
;
;*************************************************************************
;* Begin Map Control Procedures
;*************************************************************************

Declare Map_LoadMap(Map_Full_Path.s,MapType.l)
Declare Map_LoadTiles(Tile_Full_Path.s,TileWidth.l,TileHeight.l,TileSpacer.l)
Declare Map_ReadTextMapDimensions(Map_Full_Path.s)
Declare Map_LoadTextMap(Map_Full_Path.s)
Declare Map_GetTileNumber(X.l, Y.l,TileWidth.l,TileHeight.l)
Declare Map_ResetMap(X,Y)
Declare Map_ShowMap(XOffset.l,YOffset.l,Direction.l,Distance.l,ShowWidth.l,ShowHeight.l,TileWidth.l,TileHeight.l,ShowBoxes.l)
Declare Map_VerifyMapAndTiles()
Declare Map_ClearMap(Width.l,Height.l)
Declare Map_Init(Width.l,Height.l)
Declare Map_IncreaseWidth(Amount.l)
Declare Map_DecreaseWidth(Amount.l)
Declare Map_IncreaseHeight(Amount.l)
Declare Map_DecreaseHeight(Amount.l)
Declare Map_SaveBinaryMap(SaveFileName.s)
Declare Map_LoadBinaryMap(Map_Full_Path.s)
Declare Map_SetupPlayerBoundingBoxes(TopX1.l,TopY1.l,TopX2.l,TopY2.l,BottomX1.l,BottomY1.l,BottomX2.l,BottomY2.l,LeftX1.l,LeftY1.l,LeftX2.l,LeftY2.l,RightX1.l,RightY1.l,RightX2.l,RightY2.l)
Declare Map_AddWall(TileToAdd.l,TileWidth,TileHeight)
Declare Map_CheckWallCollision(X.l,Y.l,TileWidth,TileHeight)
Declare Map_CheckWallCollisionManual(X1.l,Y1.l,X2.l,Y2.l,TileWidth,TileHeight)
Declare Map_CheckWallCollisionBottom(X.l,Y.l,TileWidth,TileHeight)
Declare Map_AddLadder(TileToAdd.l,TileWidth,TileHeight)
Declare Map_CheckOnLadder(X.l,Y.l,TileWidth,TileHeight)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_LoadMap(...)
;       Author:  Krylar
;  Description: This function loads ONLY the map, no tiles!
;
; Arguments: 
;    Map_Full_Path.s = Map filename, including the full path
;    MapType.l = 0=Text, 1=Binary
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_LoadMap(Map_Full_Path.s,MapType.l)

If MapType = 0
  ; call the Map_ReadDimensions to get the proper rows/columns
  Status = Map_ReadTextMapDimensions(Map_Full_Path.s)
  
  If Status = 0
    ProcedureReturn(-2)
  EndIf  
  
  ; Now read the actual data from that map file
  Status = Map_LoadTextMap(Map_Full_Path.s)
  If Status = 0
    ProcedureReturn(-3)
  EndIf
Else
  Status = Map_LoadBinaryMap(Map_Full_Path.s)
  If Status = 0
    ProcedureReturn(-3)
  EndIf
  
EndIf
ProcedureReturn(0)
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_LoadTiles(...)
;       Author:  Krylar
;  Description: This function loads in the actual tiles to be used
;  with the map.  It takes into account any boxes that may have 
;  been placed around each image by allowing the caller to specify
;  the box widths and heights.
;
; Arguments: 
;    Tile_Full_Path.s = Tile Image file, including the full path
;    TileWidth = Width of the tiles being loaded
;    TileHeight = Height of the tiles being loaded
;    TileSpacer = If there are boxes around the tiles, set this to
;              the number of pixels that the boxes are wide and high.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_LoadTiles(Tile_Full_Path.s,TileWidth.l,TileHeight.l,TileSpacer.l)

  Dim Tile.Map_Tiles(0)
  
  ; first off let's load the full image containing all the tiles
  ; into a temporary space
  Temp_Image.l = LoadSprite(#PB_Any,Tile_Full_Path.s)
  
  ImageSizeX.l = SpriteWidth(Temp_Image.l)
  ImageSizeY.l = SpriteHeight(Temp_Image.l)
  
  TileColumns.l = ImageSizeX.l / TileWidth.l
  TileRows.l = ImageSizeY.l / TileHeight.l
  
  Dim Tile.Map_Tiles(TileRows.l * TileColumns.l)
  
  If Temp_Image.l = 0
    ProcedureReturn(-1)
  EndIf
  
  ; get the current buffer so we can restore to it
  UseBuffer(Temp_Image.l)
  
  ; setup our basic vars.  The X and Y values will be whatever the XSpacer
  ; and YSpacer values are. If they are 0, then it's assumed that there is
  ; no space between the tiles. If you look at the sample images included
  ; with this demo you'll see a white box around each image...that's why
  ; I have these spacers...we don't wanna load the boxes, just the images.
  X.l = TileSpacer.l
  Y.l = TileSpacer.l
  
  ; keep track of the number of images
  ImageNumber.l = 0
  
  ; run through the total number of rows (minus 1, of course)
  For Rows.l = 0 To TileRows.l -1
    ; and run through all the columns per row (minus 1 again)
    For Columns.l = 0 To TileColumns.l -1
      ; create a new TileList element and assign it the current
      ; ImageNumber.  Then populate the TileList element with
      ; the TileWidth, TileHeight, and the actual Tile_Image
      Tile(ImageNumber)\Width = TileWidth
      Tile(ImageNumber)\Height = TileHeight
      
      ; then we grab the image based off the size setup in CreateImage(...)
      ; from our X,Y location
      Tile(ImageNumber)\Image=GrabSprite(#PB_Any,X,Y,TileWidth,TileHeight)
      
      ; set the appropriate mask so we have transparencies
      TransparentSpriteColor(Tile(ImageNumber)\Image,TransparencyColor)
      
      ; now we add X to the TileWidth and the XSpacer to get the new X position.
      ; So, if our current X = 2, and the TileWidth = 32 and the XSpacer = 2, we'd
      ; have X = 2 + 32 + 2, or 36.  This means that the next time we call         
      ; GrabImage(...) it will start grabbing from the X position 36 (or the 36th pixel 
      ; from the left).         
      X.l = X.l + TileWidth.l + TileSpacer.l
      
      ; increment our tile counter (for the array positioning)
      ImageNumber.l = ImageNumber.l + 1
    Next
    ; we've finished with that row, so reset X back to the spacer position
    X.l = TileSpacer.l
    
    ; now we add Y to the TileHeight and the YSpacer to get the new Y position.
    ; So, if our current Y = 2, and the TileHeight = 32 and the YSpacer = 2, we'd
    ; have Y = 2 + 32 + 2, or 36.  This means that the next time we call  
    ; GrabImage(...) it will start grabbing from the Y position 36 (or the 36th pixel 
    ; from the top).         
    Y.l = Y.l + TileHeight.l + TileSpacer.l
  Next
  
  ; restore the buffer
  UseBuffer(-1)
  
  ; free the image from memory, so we don't hold memory for no reason
  FreeSprite(Temp_Image)
  
  ; reset our global tile tracker
  Map_TotalTiles.l = ImageNumber.l
  
  ProcedureReturn(0)
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_ReadTextDimensions(...)
;       Author:  Krylar
;  Description: This function loads in the actual dimensions of
;  the map file and stores the values in Map_Width and Map_Height.
;  This function is for use with TEXT files only.
; Arguments: 
;    Map_Full_Path.s = Map file, including the full path
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_ReadTextMapDimensions(Map_Full_Path.s)

  ; first thing we do is open the file using a Pointer Variable (which
  ; is "File" in this case)
  File=ReadFile(#PB_Any,Map_Full_Path.s)

  ; if the file is not found, return -1
  If File = 0
      ProcedureReturn(-1)
  EndIf

  ; Then we read the first line of the file...it *should* contain the
  ; width/height data.  If it doesn't, somebody goofed up on the layout
  ; of the file
  MapDimensions.s = ReadString(File)

  ; we need to set up some variables to parse the line.  We could just
  ; set this up to have the x,y on two lines and save some trouble from
  ; a coding perspective, but this method makes the map creation more
  ; intuitive for the user...and that's our job ;)
  EndOfString = 0
  Offset = 0

  ; while we haven't reached the end of the string
  While EndOfString = 0
    ; let's first put the current position in the string into Temp$
    ; this is just 1 character from the string cause we use the Mid$(...)
    ; command to yank out that value
    Temp.s = Mid(MapDimensions.s,Offset+1,1)
  
    ; if that character is not a comma we haven't gone past the length
    ; of the string
    If Temp.s <> "," And (Offset <= Len(MapDimensions.s))
      ; then it must be a number, so we save it in the HoldString$
      HoldString.s = HoldString.s + Temp.s
      ; otherwise, it's either gone too far or it's the comma seperator
    Else
      ; if we've gone too far, then we know that the Map_Height is done
      If Offset > Len(MapDimensions.s)
        ; so we convert the current HoldString$ to an Int and assign it
        Map_Height = Val(HoldString.s)
        ; and make sure the While loop breaks
        EndOfString = 1
      ; it must be a comma, so that means our Map_Width value is loaded
      Else
        ; convert the current HoldString$ to an int and assign it
        Map_Width = Val(HoldString.s)
        ; reset the HoldString$ to blank so we can start loading Map_Height
        HoldString.s = ""
      EndIf
    EndIf
    ; increase the string position offset by 1 (to move to the next character)
    Offset = Offset + 1
  Wend

  ; we're done, so close the file!
  CloseFile(File)

  Dim Map.Map_Data(Map_Width,Map_Height)

  Map_Init(Map_Width,Map_Height)

  ProcedureReturn(0)
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_LoadTextMap(...)
;       Author:  Krylar
;  Description: This function loads in the actual map information.
;  This function is for use with TEXT files only.
;
; Arguments: 
;    Map_Full_Path$ = Map file, including the full path
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_LoadTextMap(Map_Full_Path.s)

  ; first thing we do is open the file using a Pointer Variable (which
  ; is "File" in this case)
  File=ReadFile(#PB_Any,Map_Full_Path.s);
  ; read the Map dimensions line, but don't do anything with it...this
  ; is just to move to the map data line.  Use the Map_ReadTextDimensions(...)
  ; function for getting the actual Map_Width/Map_Height, so you can DIM
  ; the Map_Data array appropriately
  MapDimensions.s = ReadString(File);

  ; set up our vars for array placements.  The X will be for columns, and
  ; the Y will be for rows.  The EndOfFile just let's us keep track of how
  ; far into the file we've gone.
  X.l=0
  Y.l=0
  EndOfFile.l = 0

  ; do this until we reach the end of the file
  While EndOfFile = 0
  
    ; read a line of data from the file and put it in the MapLine$ string
    MapLine.s = ReadString(File);
  
    ; make sure that the length of the line is more than 1 character
    If Len(MapLine.s) > 1
    
      ; set up our EndOfString var to be 0...this will help us keep
      ; track of the current MapLine$ string position
      EndOfString.l = 0

      ; This Offset var will let us keep track of our current position
      ; in the MapLine$ string
      Offset.l = 0
    
      ; until we reach the end of the MapLine$ string
      While EndOfString.l = 0
      
        ; let's first put the current position in the string into Temp$
        ; this is just 1 character from the string cause we use the Mid$(...)
        ; command to yank out that value
        Temp.s = Mid(MapLine.s,Offset.l + 1,1)
      
        ; if that character is not a comma we haven't gone past the length
        ; of the string
        If Temp.s <> "," And (Offset.l <= Len(MapLine.s)) 
          ; then it must be a number, so we save it in the HoldString$
          HoldString.s = HoldString.s + Temp.s
          ; otherwise, it's either gone too far or it's the comma seperator
        Else
          ; if we've gone too far, then we know that this line is done
          If Offset.l > Len(MapLine.s)
            ; fill in the number of the HoldString$ by converting it to an Int
            Map(X.l,Y.l)\TileNumber = Val(HoldString.s)
            ; reset the HoldString$ to a blank
            HoldString.s = ""
            ; exit the loop for *this* line (so we can read the next one)
            EndOfString.l = 1
            ; it must be a comma, so that means this column's value is loaded
          Else
            ; fill in the number of the HoldString$ by converting it to an Int
            Map(X.l,Y.l)\TileNumber = Val(HoldString.s)
            ; reset the HoldString$ to a blank
            HoldString.s = ""
            ; Increase the X (or Column) location for the Array
            X.l = X.l + 1
          EndIf
        EndIf
        ; add one to our string position offset
        Offset.l = Offset.l + 1
      Wend

      ; set X back to 0 (so we're back to column 0)
      X.l = 0
      ; add 1 to Y (so we move down 1 row in the array)
      Y.l = Y.l + 1
    ; if we've gone past the length of the file
    Else
      ; then tell the loop to stop cause we're done!
      EndOfFile.l = 1
    EndIf
  Wend

  ; make sure to close the file!
  CloseFile(File)

  ProcedureReturn(0)
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_ResetMap(...)
;       Author:  Krylar
;  Description: Resets the map to the x,y position  
;
; Arguments: 
;       X Position of map array
;       Y Position of map array
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_ResetMap(X,Y)
  Map_X_Start = X
  Map_Y_Start = Y
  Map_X_DisplayOffset = 0
  Map_Y_DisplayOffset = 0
  Map_ClearMap(0,0)
  ForEach(Wall())
    DeleteElement(Wall())
  Next
  ForEach(Ladder())
    DeleteElement(Ladder())
  Next
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_ShowMap(...)
;       Author:  Krylar
;  Description: This function moves and displays a map starting at
; a user-defined top-left corner at whatever width/height the user
; wants.  It will move N/S/E/W directions and do so at whatever 
; distance (speed) the user needs.  
;
; Arguments: 
;    XOffset    = Where to start the left edge
;    YOffset    = Where to start the top edge
;    Direction  = 1-North, 2-South, 3-East, 4-West
;    Distance   = How far to move the map per call
;    ShowWidth  = How many columns to show (tiles)
;    ShowHeight = How many rows to show (tiles)
;    TileWidth  = How Wide are the tiles for the map
;    TileHeight = How High are the tiles for the map
;    ShowBoxes  = 0-No, 1-Show collision Points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_ShowMap(XOffset.l,YOffset.l,Direction.l,Distance.l,ShowWidth.l,ShowHeight.l,TileWidth.l,TileHeight.l,ShowBoxes.l)
  RowPosition = 0
  ColumnPosition = 0
  
  ; Which way are we scrolling the map?
  Select Direction 
    Case 1 ; North
      ; First let's make sure that we're not already on the edge
      If Map_Y_Start > 0
        ; Start counting up to zero for our offset
        Map_Y_DisplayOffset =  Map_Y_DisplayOffset + Distance
        ; if we hit or go past zero, subtract the amount over 0
        ; from the TileHeight the user passed along and decrement
        ; our Map Y starting position
        If Map_Y_DisplayOffset > 0
          Map_Y_DisplayOffset = -(TileHeight) + Distance
          Map_Y_Start = Map_Y_Start - 1    
        EndIf
        ; We want to show exactly the ShowHeight
        Map_ScrollHeight = Map_Y_Start + ShowHeight
      Else
        ; since we're already on Zero (top edge), just keep scrolling
        ; until the offset hits the edge too.  
        If Map_Y_DisplayOffset < 0
          Map_Y_DisplayOffset = Map_Y_DisplayOffset + Distance
        EndIf
        ; We want to show one over the ShowWidth here so we don't
        ; end up with blinking edges
        Map_ScrollHeight = Map_Y_Start + ShowHeight + 1
      EndIf
      
    Case 2 ; South
      ; Verify that we've not gone too far down
      If Map_Y_Start < Map_Height - ShowHeight - 1 And Map_Y_Start >= 0
        ; Count down to the AdjustedTileSize (see above)
        Map_Y_DisplayOffset = Map_Y_DisplayOffset - Distance
        ; if we hit or go past the negative size, add that amount
        ; to the Tile size sent by the user and make it negative.
        If Map_Y_DisplayOffset < -(TileHeight) 
          Map_Y_DisplayOffset = 0 - Distance
          ; Increment our Map position
          Map_Y_Start = Map_Y_Start + 1   
          ; Do another sanity check to make sure we didn't run over.
          ; if we did, drop the Map position back 1 and set the offset
          ; so it's on the edge (no more scrolling)
          If Map_Y_Start >= Map_Height - ShowHeight
            Map_Y_Start = Map_Y_Start - 1
          EndIf 
        EndIf
        ; We want to show exactly the ShowHeight
        Map_ScrollHeight = Map_Y_Start + ShowHeight
      Else
        If Map_Y_Start > 0
          ; We must already be at the point where the right edge is showing, so
          ; just scroll until the offset hits the edge too.
          Map_Y_DisplayOffset = Map_Y_DisplayOffset - Distance
          ; if we hit or go past the negative size, add that amount
          ; to the Tile size sent by the user and make it negative.
          If Map_Y_DisplayOffset < -(TileHeight) 
            Map_Y_DisplayOffset = -(TileHeight)
          EndIf
        EndIf
      EndIf
      
    Case 3 ; West (Left Arrow key)
      ; First let's make sure that we're not already on the edge
      If Map_X_Start > 0
        ; Start counting up to zero for our offset
        Map_X_DisplayOffset =  Map_X_DisplayOffset + Distance
        ; if we hit or go past zero, subtract the amount over 0
        ; from the TileWidth the user passed along and decrement
        ; our Map X starting position
        If Map_X_DisplayOffset > 0
          Map_X_DisplayOffset = -(TileWidth) + Distance
          Map_X_Start = Map_X_Start - 1    
        EndIf
        ; We want to show exactly the ShowWidth
        Map_ScrollWidth = Map_X_Start + ShowWidth
      Else
        ; since we're already on Zero (left edge), just keep scrolling
        ; until the offset hits the edge too.  
        If Map_X_DisplayOffset < 0
          Map_X_DisplayOffset = Map_X_DisplayOffset + Distance
        EndIf
        ; We want to show one over the ShowWidth here so we don't
        ; end up with blinking edges
        Map_ScrollWidth = Map_X_Start + ShowWidth + 1
      EndIf
      
    Case 4 ; East (Right Arrow Key)
      ; Verify that we've not gone too far right
      If Map_X_Start < Map_Width - ShowWidth - 1 And Map_X_Start >= 0
        ; Count down to the AdjustedTileSize (see above)
        Map_X_DisplayOffset = Map_X_DisplayOffset - Distance
        ; if we hit or go past the negative size, add that amount
        ; to the Tile size sent by the user and make it negative.
        If Map_X_DisplayOffset < -(TileWidth) 
          Map_X_DisplayOffset = 0 - Distance
          ; Increment our Map position
          Map_X_Start = Map_X_Start + 1   
          ; Do another sanity check to make sure we didn't run over.
          ; if we did, drop the Map position back 1 and set the offset
          ; so it's on the edge (no more scrolling)
          If Map_X_Start >= Map_Width - ShowWidth
            Map_X_Start = Map_X_Start - 1
          EndIf 
        EndIf
        ; We want to show exactly the ShowWidth
        Map_ScrollWidth = Map_X_Start + ShowWidth
      Else
        If Map_X_Start > 0
          ; We must already be at the point where the right edge is showing, so
          ; just scroll until the offset hits the edge too.
          Map_X_DisplayOffset = Map_X_DisplayOffset - Distance
          ; if we hit or go past the negative size, add that amount
          ; to the Tile size sent by the user and make it negative.
          If Map_X_DisplayOffset < -(TileWidth) 
            Map_X_DisplayOffset = -(TileWidth)
          EndIf
        EndIf
      EndIf
      
      ; we're not moving the map, but the map still needs to be display, so 
      ; just check where we are on the map and then display the proper width/height
      ; to avoid blinking.
    Default
      If Map_X_Start < Map_Width - ShowWidth
        Map_ScrollWidth = Map_X_Start + ShowWidth
      Else
        Map_ScrollWidth = Map_X_Start + ShowWidth - 1
      EndIf
      
      If Map_Y_Start < Map_Height - ShowHeight
        Map_ScrollHeight = Map_Y_Start + ShowHeight
      Else
        Map_ScrollHeight = Map_Y_Start + ShowHeight - 1
      EndIf
      If Distance = -1
        Map_X_DisplayOffset = 0
        Map_Y_DisplayOffset = 0
      EndIf
    EndSelect
        
    ; hook up our X, Y values to draw the images at the proper locations
    X = Map_X_DisplayOffset + XOffset
    Y = Map_Y_DisplayOffset + YOffset
        
    ; and then run through the Arrays and Draw stuff out!
    For Rows = Map_Y_Start To Map_ScrollHeight
      For Columns = Map_X_Start To Map_ScrollWidth
        TileNumber = Map(Columns,Rows)\TileNumber
        ; if the Tile isn't a -1, then draw it
        If TileNumber <> -1
          DisplayTransparentSprite(Tile(TileNumber)\Image,X,Y)
        EndIf
            
        ; if the user wants to see the collision boxes, draw them here
        If ShowBoxes.l = 1
          ForEach(Wall())
            If Wall()\TileNumber = TileNumber
              If StartDrawing(ScreenOutput())
                DrawingMode(4)
                Box(X,Y,TileWidth,TileHeight,RGB(0,255,255))
                DrawText(X + 10,Y + 10,Str(TileNumber))
                StopDrawing()
                Break
              Else
                MessageRequester("Error!", "Unable to Draw to ScreenOutput()",#PB_MessageRequester_Ok)
              EndIf
            EndIf
          Next
          ForEach(Ladder())
            If Ladder()\TileNumber = TileNumber
              If StartDrawing(ScreenOutput())
                DrawingMode(4)
                Box(X,Y,TileWidth,TileHeight,RGB(0,255,0))
                DrawText(X + 10,Y + 10,Str(TileNumber))
                StopDrawing()
                Break
              Else
                MessageRequester("Error!", "Unable to Draw to ScreenOutput()",#PB_MessageRequester_Ok)
              EndIf
            EndIf
          Next
          
        EndIf
        ; increase our X position for tile drawing
        X = X + TileWidth
      Next
      ; reset the X position and increase the Y position
      X = Map_X_DisplayOffset + XOffset
      Y = Y + TileHeight
    Next
EndProcedure
      
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_VerifyMapAndTiles(...)
;       Author:  Krylar
;  Description: This function makes sure that all the numbers 
;               within a Map corresponds to all the Tile numbers.
;               It replaces invalid numbers with -1.
;
; Arguments: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_VerifyMapAndTiles()
  Status.l = 0

  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      TileNumber.l = Map(Columns,Rows)\TileNumber
      If TileNumber.l > Map_TotalTiles.l
        Map(Columns.l,Rows.l)\TileNumber = -1
        Status.l = -1
      EndIf               
    Next
  Next
  ProcedureReturn(Status.l)
EndProcedure
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_ClearMap(...)
;       Author:  Krylar
;  Description: Clear the map, but allow it to be blanked to a 
;               particular size.
;
; Arguments: 
;        Width: How wide should the cleared map be defaulted to?
;        Height: How high should the cleared map be defaulted to?
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_ClearMap(Width.l,Height.l)
  
  Map_Width = Width.l
  Map_Height = Height.l
  
  Dim Map.Map_Data(Map_Width,Map_Height)
  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      Map(Columns.l,Rows.l)\TileNumber = -1
    Next
  Next 
EndProcedure
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_Init(...)
;       Author:  Krylar
;  Description: This function initialize a map with all -1 values
;               for elements.
;
; Arguments: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_Init(Width.l,Height.l)

  Map_Width = Width.l
  Map_Height = Height.l

  Dim Map.Map_Data(Map_Width,Map_Height)
  Dim MapHold.Map_Data(Map_Width,Map_Height)

  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      Map(Columns.l,Rows.l)\TileNumber = -1
    Next
  Next
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_IncreaseWidth(...)
;       Author:  Krylar
;  Description: This function increases the Map width by re-
;               dimensioning the Map array and copying over 
;               the appropriate number of elements.
;
; Arguments: 
;        Amount: How far to increase the width
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_IncreaseWidth(Amount.l)

  ; dimension the temporary array
  Dim MapHold.Map_Data(Map_Width,Map_Height)

  CurrentWidth.l = Map_Width
  CurrentHeight.l = Map_Height

  ; for all of the rows (minus 1)
  For Rows.l = 0 To CurrentHeight.l - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To CurrentWidth.l - 1
      MapHold(Columns.l,Rows.l)\TileNumber = Map(Columns,Rows)\TileNumber
    Next
  Next

  Map_Width = Map_Width + Amount.l
  Dim Map.Map_Data(Map_Width,Map_Height)

  ; for all of the rows (minus 1)
  For Rows.l = 0 To CurrentHeight.l - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To CurrentWidth.l - 1
      Map(Columns.l,Rows.l)\TileNumber = MapHold(Columns.l,Rows.l)\TileNumber
    Next
  Next
    
  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      If Map.Map_Data(Columns.l,Rows.l) < 0 Or Map.Map_Data(Columns.l,Rows.l) > Map_TotalTiles.l
        Map(Columns.l,Rows.l)\TileNumber = -1
      EndIf
    Next
  Next
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_DecreaseWidth(...)
;       Author:  Krylar
;  Description: This function decreases the Map width by re-
;               dimensioning the Map array and copying over only
;               the appropriate number of elements.
;
; Arguments: 
;        Amount: How far to decrease the width
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_DecreaseWidth(Amount.l)

  ; dimension the temporary array
  Dim MapHold.Map_Data(Map_Width,Map_Height)

  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      MapHold(Columns.l,Rows.l)\TileNumber = Map(Columns.l,Rows.l)\TileNumber
    Next
  Next

  Map_Width = Map_Width - Amount.l
  If Map_Width <= MapGridColums.l
    Map_Width = MapGridColums.l
  EndIf
  Dim Map.Map_Data(Map_Width,Map_Height)

  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      Map(Columns.l,Rows.l)\TileNumber = MapHold(Columns.l,Rows.l)\TileNumber
    Next
  Next
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_IncreaseHeight(...)
;       Author:  Krylar
;  Description: This function increases the Map height by re-
;               dimensioning the Map array and copying over 
;               the appropriate number of elements.
;
; Arguments: 
;        Amount: How far to increase the height
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_IncreaseHeight(Amount.l)

  ; dimension the temporary array
  Dim MapHold.Map_Data(Map_Width,Map_Height)

  CurrentWidth.l = Map_Width
  CurrentHeight.l = Map_Height

  ; for all of the rows (minus 1)
  For Rows.l = 0 To CurrentHeight.l - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To CurrentWidth.l - 1
      MapHold(Columns.l,Rows.l)\TileNumber = Map(Columns.l,Rows.l)\TileNumber
    Next
  Next

  Map_Height = Map_Height + Amount.l
  Dim Map.Map_Data(Map_Width,Map_Height)

  ; for all of the rows (minus 1)
  For Rows.l = 0 To CurrentHeight.l - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To CurrentWidth.l - 1
      Map(Columns.l,Rows.l)\TileNumber = MapHold(Columns.l,Rows.l)\TileNumber
    Next
  Next
    
  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      If Map.Map_Data(Columns.l,Rows.l) < 0 Or Map.Map_Data(Columns.l,Rows.l) > Map_TotalTiles.l
        Map(Columns.l,Rows.l)\TileNumber = -1
      EndIf
    Next
  Next
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_DecreaseHeight(...)
;       Author:  Krylar
;  Description: This function decreases the Map height by re-
;               dimensioning the Map array and copying over only
;               the appropriate number of elements.
;
; Arguments: 
;        Amount: How far to decrease the height
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_DecreaseHeight(Amount.l)

  ; dimension the temporary array
  Dim MapHold.Map_Data(Map_Width,Map_Height)

  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      MapHold(Columns.l,Rows.l)\TileNumber = Map(Columns.l,Rows.l)\TileNumber
    Next
  Next

  Map_Height = Map_Height - Amount.l
  If Map_Height <= MapGridRows.l
    Map_Height = MapGridRows.l
  EndIf
  Dim Map.Map_Data(Map_Width,Map_Height)

  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      Map(Columns.l,Rows.l)\TileNumber = MapHold(Columns.l,Rows.l)\TileNumber
    Next
  Next
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_SaveBinaryMap(...)
;       Author:  Krylar
;  Description: This function saves a map in binary format
;
; Arguments: 
;        SaveFileName$: Name to save it as
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_SaveBinaryMap(SaveFileName.s)

  ; first open the file for writing.  This *will* overwrite the
  ; existing file.
  FilePtr = CreateFile(#PB_Any,SaveFileName.s)

  ; write the Map_Width and Map_Height 
  WriteLong(FilePtr,Map_Width)
  WriteLong(FilePtr,Map_Height)

  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      ; get the tilenumber
      TileNumber.l = Map(Columns.l,Rows.l)\TileNumber
      ; write it to the file
      WriteLong(FilePtr,TileNumber.l)
    Next
  Next

  ; close the file!
  CloseFile(FilePtr)

  ProcedureReturn(0)
EndProcedure


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;     Procedure: Map_LoadBinaryMap(...)
;       Author:  Krylar
;  Description: This function loads a binary map
;
; Arguments: 
;        Map_Full_Path$: Name to load
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_LoadBinaryMap(Map_Full_Path.s)

  ; first thing we do is open the file using a Pointer Variable (which
  ; is "FilePtr" in this case)
  FilePtr.l = ReadFile(#PB_Any,Map_Full_Path.s);

  If FilePtr.l = 0
    ProcedureReturn(-1)
  EndIf

  Dim Map.Map_Data(0,0)
  
  ; read in the map dimensions
  Map_Width = ReadLong(FilePtr);
  Map_Height = ReadLong(FilePtr);

  Dim Map.Map_Data(Map_Width,Map_Height)

  ; set up our vars for array placements.  The X will be for columns, and
  ; the Y will be for rows.  The EndOfFile just let's us keep track of how
  ; far into the file we've gone.
  X.l = 0
  Y.l = 0
  EndOfFile.l = 0

  ; for all of the rows (minus 1)
  For Rows.l = 0 To Map_Height - 1
    ; and for all the columns (minus 1)
    For Columns.l = 0 To Map_Width - 1
      ; read in the encrypted value
      Map(X.l,Y.l)\TileNumber = ReadLong(FilePtr)
      
      ; add 1 to X (so we move 1 column to the right in the array)
      X.l = X.l + 1
    Next
    ; set X back to 0 (so we're back to column 0)
    X.l = 0
    ; add 1 to Y (so we move down 1 row in the array)
    Y.l = Y.l + 1
  Next

  ; make sure to close the file!
  CloseFile(FilePtr)

  ProcedureReturn(0)
EndProcedure

;*************************************************************************
;* End Map Control Procedures
;*************************************************************************
;
;
;
;*************************************************************************
;* Begin Collision Control Procedures                                                  
;*************************************************************************

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Map_SetupPlayerBoundingBoxes()
; Author: Krylar
;
; sets up where the x1,y1,x2,y2 values are for the 4 boxes
; that make up the collision points on the player
;
; Arguments:
;          Top, Bottom, Left, Right values of X1, Y1, X2, Y2 - Respectively
;
; Returns: n/a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_SetupPlayerBoundingBoxes(TopX1.l,TopY1.l,TopX2.l,TopY2.l,BottomX1.l,BottomY1.l,BottomX2.l,BottomY2.l,LeftX1.l,LeftY1.l,LeftX2.l,LeftY2.l,RightX1.l,RightY1.l,RightX2.l,RightY2.l)
  ; setup the top-box
  Map_PlayerCollisionArray(0) = TopX1.l
  Map_PlayerCollisionArray(1) = TopY1.l
  Map_PlayerCollisionArray(2) = TopX2.l
  Map_PlayerCollisionArray(3) = TopY2.l
  ; setup the bottom-box
  Map_PlayerCollisionArray(4) = BottomX1.l
  Map_PlayerCollisionArray(5) = BottomY1.l
  Map_PlayerCollisionArray(6) = BottomX2.l
  Map_PlayerCollisionArray(7) = BottomY2.l
  ; setup the Left-box
  Map_PlayerCollisionArray(8) = LeftX1.l
  Map_PlayerCollisionArray(9) = LeftY1.l
  Map_PlayerCollisionArray(10) = LeftX2.l
  Map_PlayerCollisionArray(11) = LeftY2.l
  ; setup the Right-box
  Map_PlayerCollisionArray(12) = RightX1.l
  Map_PlayerCollisionArray(13) = RightY1.l
  Map_PlayerCollisionArray(14) = RightX2.l
  Map_PlayerCollisionArray(15) = RightY2.l
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Map_ShowPlayerBoundingBoxes()
; Author: Krylar
;
; Gives a graphical representation of where the player is being checked
; for collisions
;
; Arguments:
;          X,Y location of the player
;
; Returns: n/a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_ShowPlayerBoundingBoxes(X.l,Y.l)
  If StartDrawing(ScreenOutput())
     DrawingMode(4) 
     Box(X+Map_PlayerCollisionArray(0),Y+Map_PlayerCollisionArray(1),Map_PlayerCollisionArray(2) - Map_PlayerCollisionArray(0),Map_PlayerCollisionArray(3) - Map_PlayerCollisionArray(1),RGB(255,255,0))
     Box(X+Map_PlayerCollisionArray(4),Y+Map_PlayerCollisionArray(5),Map_PlayerCollisionArray(6)-Map_PlayerCollisionArray(4),Map_PlayerCollisionArray(7)-Map_PlayerCollisionArray(5),RGB(255,255,0))
     Box(X+Map_PlayerCollisionArray(8),Y+Map_PlayerCollisionArray(9),Map_PlayerCollisionArray(10)-Map_PlayerCollisionArray(8),Map_PlayerCollisionArray(11)-Map_PlayerCollisionArray(9),RGB(255,255,0))
     Box(X+Map_PlayerCollisionArray(12),Y+Map_PlayerCollisionArray(13),Map_PlayerCollisionArray(14)-Map_PlayerCollisionArray(12),Map_PlayerCollisionArray(15)-Map_PlayerCollisionArray(13),RGB(255,255,0))
  Else
    MessageRequester("Error!", "Unable to Draw to ScreenOutput()",#PB_MessageRequester_Ok)
  EndIf
  StopDrawing()
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Map_AddWall()
; Author: Krylar
;
; Adds a wall to the collision list
;
; Returns: n/a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_AddWall(TileToAdd.l,TileWidth,TileHeight)
  X.l = 0
  Y.l = 0

  ; run through all of the map
  For Rows.l = 0 To Map_Height - 1
    For Columns.l = 0 To Map_Width - 1
      TileNumber.l = Map(Columns.l,Rows.l)\TileNumber
      ; see if the tile number is one we want designated as a wall
      If TileNumber.l = TileToAdd.l
        ; Add it to the Wall Type
        AddElement(Wall())
        Wall()\TileNumber = TileNumber.l
        Wall()\X = X.l
        Wall()\Y = Y.l
      EndIf
      X.l = X.l + TileWidth 
    Next
    ; reset X to the beginning of the next row
    X.l = 0
    ; increase our Y drawing position by the Tile's Height
    Y.l = Y.l + TileHeight
  Next
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Map_CheckWallCollisions(...)
; Author: Krylar
;
; Checks an area for collision based on bounding boxes
;
; Returns:
;           0 = no hit
;           1 = collision with wall
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_CheckWallCollision(X.l,Y.l,TileWidth,TileHeight)

  ; do one calculation here for each absolute box position
  ; so we don't do them every itteration of our loop below
  BoxTopX1.l    = X.l + Map_PlayerCollisionArray(0)
  BoxTopY1.l    = Y.l + Map_PlayerCollisionArray(1)
  BoxTopX2.l    = X.l + Map_PlayerCollisionArray(2)
  BoxTopY2.l    = Y.l + Map_PlayerCollisionArray(3)
  BoxBottomX1.l = X.l + Map_PlayerCollisionArray(4)
  BoxBottomY1.l = Y.l + Map_PlayerCollisionArray(5)
  BoxBottomX2.l = X.l + Map_PlayerCollisionArray(6)
  BoxBottomY2.l = Y.l + Map_PlayerCollisionArray(7)
  BoxLeftX1.l   = X.l + Map_PlayerCollisionArray(8)
  BoxLeftY1.l   = Y.l + Map_PlayerCollisionArray(9)
  BoxLeftX2.l   = X.l + Map_PlayerCollisionArray(10)
  BoxLeftY2.l   = Y.l + Map_PlayerCollisionArray(11)
  BoxRightX1.l  = X.l + Map_PlayerCollisionArray(12)
  BoxRightY1.l  = Y.l + Map_PlayerCollisionArray(13)
  BoxRightX2.l  = X.l + Map_PlayerCollisionArray(14)
  BoxRightY2.l  = Y.l + Map_PlayerCollisionArray(15)
  
  
  ; run through all of the walls
  ForEach(Wall())
    XCollision.l = 0
    YCollision.l = 0
  
    ; grab the Type values to speed things up a bit
    WallX.l = Wall()\X
    WallY.l = Wall()\Y

    ; calculate the WallWidth and Height to speed things up
    WallWidth.l = WallX.l + TileWidth 
    WallHeight.l = WallY.l + TileHeight
  
    ; check the top bounding box
    If BoxTopX1.l >= WallX.l And BoxTopX2.l <= WallWidth.l
      XCollision.l = 1
    EndIf
    If BoxTopY1.l >= WallY.l And BoxTopY2.l <= WallHeight.l
      YCollision.l = 1
    EndIf
  
    ; check the bottom bounding box
    If BoxBottomX1.l >= WallX.l And BoxBottomX2.l <= WallWidth.l
      XCollision.l = 1
    EndIf
    If BoxBottomY1.l >= WallY.l And BoxBottomY2.l <= WallHeight.l
      YCollision.l = 1
    EndIf
  
    ; check the left bounding box
    If BoxLeftX1.l >= WallX.l And BoxLeftX2.l <= WallWidth.l
      XCollision.l = 1
    EndIf
    If BoxLeftY1.l >= WallY.l And BoxLeftY2.l <= WallHeight.l
      YCollision.l = 1
    EndIf
  
    ; check the right bounding box
    If BoxRightX1.l >= WallX.l And BoxRightX2.l <= WallWidth.l
      XCollision.l = 1
    EndIf
    If BoxRightY1.l >= WallY.l And BoxRightY2.l <= WallHeight.l
      YCollision.l = 1
    EndIf
  
    ; if there is a collision on both the X and Y axis, return 1
    If XCollision.l = 1 And YCollision.l = 1
      ProcedureReturn(1)
    EndIf
  Next
   
  ; no hit so return 0
  ProcedureReturn(0)
EndProcedure


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Map_CheckWallCollisionsManual(...)
; Author: Krylar
;
; Checks an area for collision based on bounding boxes
;
; Returns:
;           0 = no hit
;           1 = collision with wall
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_CheckWallCollisionManual(X1.l,Y1.l,X2.l,Y2.l,TileWidth,TileHeight)
  ; run through all of the walls
  ForEach(Wall())
    XCollision.l = 0
    YCollision.l = 0
  
    ; grab the Type values to speed things up a bit
    WallX.l = Wall()\X
    WallY.l = Wall()\Y

    ; calculate the WallWidth and Height to speed things up
    WallWidth.l = WallX.l + TileWidth 
    WallHeight.l = WallY.l + TileHeight
  
    ; check the top bounding box
    If X1.l >= WallX.l And X2.l <= WallWidth.l
      XCollision.l = 1
    EndIf
    If Y1.l >= WallY.l And Y2.l <= WallHeight.l
      YCollision.l = 1
    EndIf
  
    ; if there is a collision on both the X and Y axis, return 1
    If XCollision.l = 1 And YCollision.l = 1
      ProcedureReturn(1)
    EndIf
  Next
   
  ; no hit so return 0
  ProcedureReturn(0)
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Map_CheckWallCollisionBottom(...)
; Author: Krylar
;
; Checks an area for collision based on the bottom bounding box
;
; Returns:
;           0 = no hit
;           1 = collision with wall
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_CheckWallCollisionBottom(X.l,Y.l,TileWidth,TileHeight)
  
  ; do one calculation here for each absolute box position
  ; so we don't do them every itteration of our loop below
  BoxBottomX1.l = X.l + Map_PlayerCollisionArray(4)
  BoxBottomY1.l = Y.l + Map_PlayerCollisionArray(5)
  BoxBottomX2.l = X.l + Map_PlayerCollisionArray(6)
  BoxBottomY2.l = Y.l + Map_PlayerCollisionArray(7)
  
  ; run through all of the walls
  ForEach(Wall())
    XCollision.l = 0
    YCollision.l = 0
    
    ; grab the Type values to speed things up a bit
    WallX.l = Wall()\X
    WallY.l = Wall()\Y
    
    ; calculate the WallWidth and Height to speed things up
    WallWidth.l = WallX.l + TileWidth 
    WallHeight.l = WallY.l + TileHeight
    
    ; check the bottom bounding box
    If BoxBottomX1.l >= WallX.l And BoxBottomX2.l <= WallWidth.l
      XCollision.l = 1
    EndIf
    If BoxBottomY1.l >= WallY.l And BoxBottomY2.l <= WallHeight.l
      YCollision.l = 1
    EndIf
    
    ; if there is a collision on both the X and Y axis, return 1
    If XCollision.l = 1 And YCollision.l = 1
      ProcedureReturn(1)
    EndIf
  Next
  
  ; no hit so return 0
  ProcedureReturn(0)
EndProcedure


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Map_AddLadder()
; Author: Krylar
;
; Adds a ladder to the collision list
;
; Returns: n/a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_AddLadder(TileToAdd.l,TileWidth,TileHeight)
  X.l = 0
  Y.l = 0
  
  ; run through all of the map
  For Rows.l = 0 To Map_Height - 1
    For Columns.l = 0 To Map_Width - 1
      TileNumber.l = Map(Columns.l,Rows.l)\TileNumber
      ; see if the tile number is one we want designated as a ladder
      If TileNumber.l = TileToAdd.l
        ; Add it to the ladder Type
        AddElement(Ladder())
        Ladder()\TileNumber = TileNumber.l
        Ladder()\X = X.l
        Ladder()\Y = Y.l
      EndIf
      X.l = X.l + TileWidth 
    Next
    ; reset X to the beginning of the next row
    X.l = 0
    ; increase our Y drawing position by the Tile's Height
    Y.l = Y.l + TileHeight
  Next
EndProcedure


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure: Map_CheckOnLadder(...)
; Author: Krylar
;
; Checks an area for collision based on bounding boxes
;
; Returns:
;           0 = no hit
;           1 = collision with ladder
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure Map_CheckOnLadder(X.l,Y.l,TileWidth,TileHeight)
  
  ; do one calculation here for each absolute box position
  ; so we don't do them every itteration of our loop below
  BoxTopY1.l    = Y.l + Map_PlayerCollisionArray(1)
  BoxLeftX1.l   = X.l + Map_PlayerCollisionArray(8)
  BoxRightX2.l  = X.l + Map_PlayerCollisionArray(14)
  BoxBottomY2.l = Y.l + Map_PlayerCollisionArray(7)
  
  
  ; run through all of the ladder
  ForEach(Ladder())
    XCollision.l = 0
    YCollision.l = 0
    
    ; grab the Type values to speed things up a bit
    LadderX.l = Ladder()\X
    LadderY.l = Ladder()\Y
    
    ; calculate the WallWidth and Height to speed things up
    LadderWidth.l = LadderX.l + TileWidth 
    LadderHeight.l = LadderY.l + TileHeight
    
    ; check the left bounding box
    If BoxLeftX1.l >= LadderX.l And BoxRightX2.l <= LadderWidth.l
      XCollision.l = 1
    EndIf
    If (BoxTopY1.l > LadderY.l And BoxTopY1.l < LadderHeight.l) Or (BoxBottomY2.l > LadderY.l And BoxBottomY2.l < LadderHeight.l)
      YCollision.l = 1
    EndIf
    
    ; if there is a collision on both the X and Y axis, return 1
    If XCollision.l = 1 And YCollision.l = 1
      ProcedureReturn(1)
    EndIf
  Next
  
  ; no hit so return 0
  ProcedureReturn(0)
EndProcedure



;*************************************************************************
;* End Collision Control Procedures                                                  
;*************************************************************************

; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 1352
; FirstLine = 1339
; Folding = ----